@@ -20,6 +20,10 @@
 #include <my_sys.h>                             // pthread_mutex_t
 #include "m_string.h"                           // LEX_CUSTRING
 
+#define ERROR_INJECT(code) \
+  ((DBUG_IF("crash_" code) && (DBUG_SUICIDE(), 0)) || \
+   (DBUG_IF("fail_" code) && (my_error(ER_UNKNOWN_ERROR, MYF(0)), 1)))
+
 class Alter_info;
 class Alter_table_ctx;
 class Column_definition;
@@ -53,6 +57,8 @@ enum enum_explain_filename_mode
 #define WFRM_WRITE_SHADOW 1
 #define WFRM_INSTALL_SHADOW 2
 #define WFRM_KEEP_SHARE 4
+#define WFRM_WRITE_CONVERTED_TO 8
+#define WFRM_BACKUP_ORIGINAL 16
 
 /* Flags for conversion functions. */
 static const uint FN_FROM_IS_TMP=  1 << 0;
@@ -77,7 +83,8 @@ bool check_mysql50_prefix(const char *name);
 uint build_table_filename(char *buff, size_t bufflen, const char *db,
                           const char *table, const char *ext, uint flags);
 uint build_table_shadow_filename(char *buff, size_t bufflen,
-                                 ALTER_PARTITION_PARAM_TYPE *lpt);
+                                 ALTER_PARTITION_PARAM_TYPE *lpt,
+                                 bool backup= false);
 void build_lower_case_table_filename(char *buff, size_t bufflen,
                                      const LEX_CSTRING *db,
                                      const LEX_CSTRING *table,
@@ -128,16 +135,24 @@ bool add_keyword_to_query(THD *thd, String *result, const LEX_CSTRING *keyword,
 int mysql_create_table_no_lock(THD *thd,
                                DDL_LOG_STATE *ddl_log_state,
                                DDL_LOG_STATE *ddl_log_state_rm,
+                               const LEX_CSTRING *db,
+                               const LEX_CSTRING *table_name,
                                Table_specification_st *create_info,
                                Alter_info *alter_info, bool *is_trans,
                                int create_table_mode, TABLE_LIST *table);
 
-handler *mysql_create_frm_image(THD *thd, HA_CREATE_INFO *create_info,
-                                Alter_info *alter_info, int create_table_mode,
-                                KEY **key_info, uint *key_count,
+handler *mysql_create_frm_image(THD *thd,
+                                const LEX_CSTRING &db,
+                                const LEX_CSTRING &table_name,
+                                HA_CREATE_INFO *create_info,
+                                Alter_info *alter_info,
+                                int create_table_mode,
+                                KEY **key_info,
+                                uint *key_count,
                                 LEX_CUSTRING *frm);
 
-int mysql_discard_or_import_tablespace(THD *thd, TABLE_LIST *table_list,
+int mysql_discard_or_import_tablespace(THD *thd,
+                                       TABLE_LIST *table_list,
                                        bool discard);
 
 bool mysql_prepare_alter_table(THD *thd, TABLE *table,
@@ -198,7 +213,8 @@ int write_bin_log(THD *thd, bool clear_error,
                   char const *query, ulong query_length,
                   bool is_trans= FALSE);
 int write_bin_log_with_if_exists(THD *thd, bool clear_error,
-                                 bool is_trans, bool add_if_exists);
+                                 bool is_trans, bool add_if_exists,
+                                 bool commit_alter= false);
 
 void promote_first_timestamp_column(List<Create_field> *column_definitions);
 
