@@ -304,13 +304,13 @@ Datafile::read_first_page(bool read_only_mode)
 					      + m_first_page);
 		m_flags = fsp_header_get_flags(m_first_page);
 		if (!fil_space_t::is_valid_flags(m_flags, m_space_id)) {
-			ulint cflags = fsp_flags_convert_from_101(m_flags);
-			if (cflags == ULINT_UNDEFINED) {
+			uint32_t cflags = fsp_flags_convert_from_101(m_flags);
+			if (cflags == UINT32_MAX) {
 				switch (fsp_flags_is_incompatible_mysql(m_flags)) {
 				case 0:
-					sql_print_error("InnoDB: Invalid flags 0x%zx in %s",
+					sql_print_error("InnoDB: Invalid flags 0x%" PRIx32 " in %s",
 							m_flags, m_filepath);
-					return(DB_CORRUPTION);
+					return DB_CORRUPTION;
 				case 3:
 				case 2:
 					sql_print_error("InnoDB: MySQL-8.0 tablespace in %s",
@@ -322,7 +322,7 @@ Datafile::read_first_page(bool read_only_mode)
 					break;
 				}
 				sql_print_error("InnoDB: Restart in MySQL for migration/recovery.");
-				return(DB_UNSUPPORTED);
+				return DB_UNSUPPORTED;
 			} else {
 				m_flags = cflags;
 			}
@@ -355,8 +355,7 @@ in order for this function to validate it.
 @param[in]	flags		The expected tablespace flags.
 @retval DB_SUCCESS if tablespace is valid, DB_ERROR if not.
 m_is_valid is also set true on success, else false. */
-dberr_t
-Datafile::validate_to_dd(ulint space_id, ulint flags)
+dberr_t Datafile::validate_to_dd(uint32_t space_id, uint32_t flags)
 {
 	dberr_t err;
 
@@ -367,7 +366,7 @@ Datafile::validate_to_dd(ulint space_id, ulint flags)
 	/* Validate this single-table-tablespace with the data dictionary,
 	but do not compare the DATA_DIR flag, in case the tablespace was
 	remotely located. */
-	err = validate_first_page(0);
+	err = validate_first_page();
 	if (err != DB_SUCCESS) {
 		return(err);
 	}
@@ -410,7 +409,7 @@ Datafile::validate_for_recovery()
 	ut_ad(is_open());
 	ut_ad(!srv_read_only_mode);
 
-	err = validate_first_page(0);
+	err = validate_first_page();
 
 	switch (err) {
 	case DB_TABLESPACE_EXISTS:
@@ -436,39 +435,28 @@ Datafile::validate_for_recovery()
 			return(err);
 		}
 
-		if (!m_space_id) {
-			m_space_id = recv_sys.dblwr.find_first_page(
-				m_filepath, m_handle);
-			if (m_space_id) {
-				m_defer= false;
-				goto free_first_page;
-			} else return err;
-		}
-
 		if (!m_defer) {
 			err = find_space_id();
 			if (err != DB_SUCCESS || m_space_id == 0) {
-				sql_print_error(
-					"InnoDB: Datafile '%s' is corrupted."
-					" Cannot determine the space ID from"
-					" the first 64 pages.", m_filepath);
+				ib::error() << "Datafile '" << m_filepath
+					<< "' is corrupted. Cannot determine "
+					"the space ID from the first 64 pages.";
 				return(err);
 			}
 		}
 
-		if (m_space_id == ULINT_UNDEFINED) {
+		if (m_space_id == UINT32_MAX) {
 			return DB_SUCCESS; /* empty file */
 		}
 
-		if (recv_sys.dblwr.restore_first_page(
-			m_space_id, m_filepath, m_handle)) {
+		if (restore_from_doublewrite()) {
 			return m_defer ? err : DB_CORRUPTION;
 		}
-free_first_page:
+
 		/* Free the previously read first page and then re-validate. */
 		free_first_page();
 		m_defer = false;
-		err = validate_first_page(0);
+		err = validate_first_page();
 	}
 
 	return(err);
@@ -478,11 +466,10 @@ free_first_page:
 tablespace is opened.  This occurs before the fil_space_t is created
 so the Space ID found here must not already be open.
 m_is_valid is set true on success, else false.
-@param[out]	flush_lsn	contents of FIL_PAGE_FILE_FLUSH_LSN
 @retval DB_SUCCESS on if the datafile is valid
 @retval DB_CORRUPTION if the datafile is not readable
 @retval DB_TABLESPACE_EXISTS if there is a duplicate space_id */
-dberr_t Datafile::validate_first_page(lsn_t *flush_lsn)
+dberr_t Datafile::validate_first_page()
 {
 	const char*	error_txt = NULL;
 
@@ -492,14 +479,6 @@ dberr_t Datafile::validate_first_page(lsn_t *flush_lsn)
 	    && read_first_page(srv_read_only_mode) != DB_SUCCESS) {
 
 		error_txt = "Cannot read first page";
-	} else {
-		ut_ad(m_first_page);
-
-		if (flush_lsn != NULL) {
-
-			*flush_lsn = mach_read_from_8(
-				m_first_page + FIL_PAGE_FILE_FLUSH_LSN_OR_KEY_VERSION);
-		}
 	}
 
 	if (error_txt != NULL) {
@@ -512,12 +491,11 @@ err_exit:
 			return DB_SUCCESS;
 		}
 
-		sql_print_information(
-				"InnoDB: %s in datafile: %s, Space ID: %zu, "
-				"Flags: %zu", error_txt, m_filepath,
-				m_space_id, m_flags);
+		ib::info() << error_txt << " in datafile: " << m_filepath
+			<< ", Space ID:" << m_space_id  << ", Flags: "
+			<< m_flags;
 		m_is_valid = false;
-		return DB_CORRUPTION;
+		return(DB_CORRUPTION);
 	}
 
 	/* Check if the whole page is blank. */
@@ -651,15 +629,15 @@ Datafile::find_space_id()
 	     page_size <<= 1) {
 		/* map[space_id] = count of pages */
 		typedef std::map<
-			ulint,
-			ulint,
-			std::less<ulint>,
-			ut_allocator<std::pair<const ulint, ulint> > >
+			uint32_t,
+			uint32_t,
+			std::less<uint32_t>,
+			ut_allocator<std::pair<const uint32_t, uint32_t> > >
 			Pages;
 
 		Pages	verify;
-		ulint	page_count = 64;
-		ulint	valid_pages = 0;
+		uint32_t page_count = 64;
+		uint32_t valid_pages = 0;
 
 		/* Adjust the number of pages to analyze based on file size */
 		while ((page_count * page_size) > file_size) {
@@ -673,14 +651,14 @@ Datafile::find_space_id()
 		byte*	page = static_cast<byte*>(
 			aligned_malloc(page_size, page_size));
 
-		ulint fsp_flags;
+		uint32_t fsp_flags;
 		/* provide dummy value if the first os_file_read() fails */
 		switch (srv_checksum_algorithm) {
 		case SRV_CHECKSUM_ALGORITHM_STRICT_FULL_CRC32:
 		case SRV_CHECKSUM_ALGORITHM_FULL_CRC32:
 			fsp_flags = 1U << FSP_FLAGS_FCRC32_POS_MARKER
 				| FSP_FLAGS_FCRC32_PAGE_SSIZE()
-				| innodb_compression_algorithm
+				| uint(innodb_compression_algorithm)
 				       << FSP_FLAGS_FCRC32_POS_COMPRESSED_ALGO;
 			break;
 		default:
@@ -720,7 +698,7 @@ Datafile::find_space_id()
 
 			if (noncompressed_ok || compressed_ok) {
 
-				ulint	space_id = mach_read_from_4(page
+				uint32_t space_id = mach_read_from_4(page
 					+ FIL_PAGE_SPACE_ID);
 
 				if (space_id > 0) {
@@ -771,6 +749,60 @@ Datafile::find_space_id()
 	return(DB_CORRUPTION);
 }
 
+
+/** Restore the first page of the tablespace from
+the double write buffer.
+@return whether the operation failed */
+bool
+Datafile::restore_from_doublewrite()
+{
+	if (srv_operation > SRV_OPERATION_EXPORT_RESTORED) {
+		return true;
+	}
+
+	/* Find if double write buffer contains page_no of given space id. */
+	const page_id_t	page_id(m_space_id, 0);
+	const byte*	page = recv_sys.dblwr.find_page(page_id);
+
+	if (!page) {
+		/* If the first page of the given user tablespace is not there
+		in the doublewrite buffer, then the recovery is going to fail
+		now. Hence this is treated as an error. */
+
+		ib::error()
+			<< "Corrupted page " << page_id
+			<< " of datafile '" << m_filepath
+			<< "' could not be found in the doublewrite buffer.";
+		return(true);
+	}
+
+	uint32_t flags = mach_read_from_4(
+		FSP_HEADER_OFFSET + FSP_SPACE_FLAGS + page);
+
+	if (!fil_space_t::is_valid_flags(flags, m_space_id)) {
+		flags = fsp_flags_convert_from_101(flags);
+		/* recv_dblwr_t::validate_page() inside find_page()
+		checked this already. */
+		ut_ad(flags != UINT32_MAX);
+		/* The flags on the page should be converted later. */
+	}
+
+	ulint physical_size = fil_space_t::physical_size(flags);
+
+	ut_a(page_get_page_no(page) == page_id.page_no());
+
+	ib::info() << "Restoring page " << page_id
+		<< " of datafile '" << m_filepath
+		<< "' from the doublewrite buffer. Writing "
+		<< ib::bytes_iec{physical_size} << " into file '"
+		<< m_filepath << "'";
+
+	return(os_file_write(
+			IORequestWrite,
+			m_filepath, m_handle, page, 0, physical_size)
+	       != DB_SUCCESS);
+}
+
 /** Read an InnoDB Symbolic Link (ISL) file by name.
 @param link_filepath   filepath of the ISL file
 @return data file name (must be freed by the caller)
