@@ -14,7 +14,10 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
-#include "hs_compat.h"
+#define SPD_INIT_DYNAMIC_ARRAY2(A, B, C, D, E, F)                             \
+  my_init_dynamic_array2(PSI_INSTRUMENT_ME, A, B, C, D, E, F)
+#define SPD_INIT_ALLOC_ROOT(A, B, C, D)                                       \
+  init_alloc_root(PSI_INSTRUMENT_ME, A, B, C, D)
 
 #define SPIDER_DBTON_SIZE 15
 
@@ -25,31 +28,56 @@
 #define SPIDER_DB_WRAPPER_MYSQL "mysql"
 #define SPIDER_DB_WRAPPER_MARIADB "mariadb"
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100204
 #define PLUGIN_VAR_CAN_MEMALLOC
 /*
+#define ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
 #define HASH_UPDATE_WITH_HASH_VALUE
 */
+#else
+#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS
+#define HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
+#endif
+#endif
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100002
 #define SPIDER_HAS_DISCOVER_TABLE_STRUCTURE
 #define SPIDER_HAS_APPEND_FOR_SINGLE_QUOTE
 #define SPIDER_HAS_SHOW_SIMPLE_FUNC
 #define SPIDER_HAS_JT_HASH_INDEX_MERGE
 #define SPIDER_HAS_EXPR_CACHE_ITEM
+#else
+#define SPIDER_NEED_CHECK_CONDITION_AT_CHECKING_DIRECT_ORDER_LIMIT
+#endif
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100007
 #define SPIDER_ITEM_HAS_CMP_TYPE
+#endif
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100004
 #define SPIDER_HAS_TIME_STATUS
 #define SPIDER_HAS_DECIMAL_OPERATION_RESULTS_VALUE_TYPE
+#endif
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100014
 #define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100100
 #define SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY_AND_THDPTR
+#endif
+#endif
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100108
 #define SPIDER_HAS_GROUP_BY_HANDLER
+#endif
 
+#if defined(MARIADB_BASE_VERSION) && MYSQL_VERSION_ID >= 100200
 #define SPIDER_ORDER_HAS_ENUM_ORDER
+#endif
 
+#if defined(MARIADB_BASE_VERSION)
 #define SPIDER_ITEM_GEOFUNC_NAME_HAS_MBR
 #define SPIDER_HANDLER_AUTO_REPAIR_HAS_ERROR
+#endif
 
 class spider_db_conn;
 typedef spider_db_conn SPIDER_DB_CONN;
@@ -91,6 +119,7 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_HS_LTEQUAL_STR "<="
 #define SPIDER_SQL_HS_LTEQUAL_LEN (sizeof(SPIDER_SQL_HS_LTEQUAL_STR) - 1)
 
+#ifdef ITEM_FUNC_CASE_PARAMS_ARE_PUBLIC
 #define SPIDER_SQL_CASE_STR "case "
 #define SPIDER_SQL_CASE_LEN (sizeof(SPIDER_SQL_CASE_STR) - 1)
 #define SPIDER_SQL_WHEN_STR " when "
@@ -101,6 +130,7 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_ELSE_LEN (sizeof(SPIDER_SQL_ELSE_STR) - 1)
 #define SPIDER_SQL_END_STR " end"
 #define SPIDER_SQL_END_LEN (sizeof(SPIDER_SQL_END_STR) - 1)
+#endif
 
 #define SPIDER_SQL_USING_STR " using "
 #define SPIDER_SQL_USING_LEN (sizeof(SPIDER_SQL_USING_STR) - 1)
@@ -192,6 +222,11 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_TYPE_DROP_TMP_TABLE_SQL (1 << 6)
 #define SPIDER_SQL_TYPE_OTHER_SQL (1 << 7)
 #define SPIDER_SQL_TYPE_HANDLER (1 << 8)
+#define SPIDER_SQL_TYPE_SELECT_HS (1 << 9)
+#define SPIDER_SQL_TYPE_INSERT_HS (1 << 10)
+#define SPIDER_SQL_TYPE_UPDATE_HS (1 << 11)
+#define SPIDER_SQL_TYPE_DELETE_HS (1 << 12)
+#define SPIDER_SQL_TYPE_OTHER_HS (1 << 13)
 
 enum spider_bulk_upd_start {
   SPD_BU_NOT_START,
@@ -392,6 +427,12 @@ public:
     IO_CACHE *file,
     uint32 arg_length
   );
+  bool append_with_prefill(
+    const char *s,
+    uint32 arg_length,
+    uint32 full_length,
+    char fill_char
+  );
   int strstr(
     const String &search,
     uint32 offset = 0
@@ -551,26 +592,34 @@ typedef struct spider_conn_holder
   spider_conn_holder *next;
 } SPIDER_CONN_HOLDER;
 
-/* Record information of a local (spider) table, for use of the spider
-group by handler. */
 typedef struct spider_table_holder
 {
   TABLE *table;
   ha_spider *spider;
-  /* alias of the table, in the form of tk, where k is the index of
-  the table from `query->from' indexed by next_local. */
   spider_string *alias;
 } SPIDER_TABLE_HOLDER;
 
-/* For use of the spider group by handler. */
+typedef struct spider_field_holder
+{
+  Field *field;
+  ha_spider *spider;
+  spider_string *alias;
+  spider_field_holder *next;
+} SPIDER_FIELD_HOLDER;
+
+typedef struct spider_field_chain
+{
+  spider_field_holder *field_holder;
+  spider_field_chain *next;
+} SPIDER_FIELD_CHAIN;
+
 class spider_fields
 {
   uint dbton_count;
   uint current_dbton_num;
   uint dbton_ids[SPIDER_DBTON_SIZE];
-  /* Number of tables in `query->from'. */
   uint table_count;
-  /* All tables in `query->from', in the same order by next_local. */
+  uint current_table_num;
   SPIDER_TABLE_HOLDER *table_holder;
   SPIDER_LINK_IDX_CHAIN *first_link_idx_chain;
   SPIDER_LINK_IDX_CHAIN *last_link_idx_chain;
@@ -579,6 +628,13 @@ class spider_fields
   SPIDER_CONN_HOLDER *first_conn_holder;
   SPIDER_CONN_HOLDER *last_conn_holder;
   SPIDER_CONN_HOLDER *current_conn_holder;
+  SPIDER_FIELD_HOLDER *first_field_holder;
+  SPIDER_FIELD_HOLDER *last_field_holder;
+  SPIDER_FIELD_HOLDER *current_field_holder;
+  SPIDER_FIELD_CHAIN *first_field_chain;
+  SPIDER_FIELD_CHAIN *last_field_chain;
+  SPIDER_FIELD_CHAIN *current_field_chain;
+  Field **first_field_ptr;
   Field **current_field_ptr;
 public:
   spider_fields();
@@ -621,6 +677,8 @@ public:
     long access_balance
   );
   SPIDER_CONN_HOLDER *create_conn_holder();
+  void set_pos_to_first_conn_holder();
+  SPIDER_CONN_HOLDER *get_next_conn_holder();
   bool has_conn_holder();
   void clear_conn_holder_from_conn();
   bool check_conn_same_conn(
@@ -634,14 +692,24 @@ public:
   void free_conn_holder(
     SPIDER_CONN_HOLDER *conn_holder_arg
   );
-  SPIDER_TABLE_HOLDER *find_table(Field *field);
-  void set_table_holder(
-    SPIDER_TABLE_HOLDER *table_holder_arg,
+  SPIDER_TABLE_HOLDER *add_table(
+    ha_spider *spider_arg
+  );
+  bool all_query_fields_are_query_table_members();
+  int create_table_holder(
     uint table_count_arg
   );
-  SPIDER_TABLE_HOLDER *get_first_table_holder();
+  void set_pos_to_first_table_holder();
+  SPIDER_TABLE_HOLDER *get_next_table_holder();
   SPIDER_TABLE_HOLDER *get_table_holder(TABLE *table);
   uint get_table_count();
+  int add_field(Field *field_arg);
+  SPIDER_FIELD_HOLDER *create_field_holder();
+  void set_pos_to_first_field_holder();
+  SPIDER_FIELD_HOLDER *get_next_field_holder();
+  SPIDER_FIELD_CHAIN *create_field_chain();
+  void set_pos_to_first_field_chain();
+  SPIDER_FIELD_CHAIN *get_next_field_chain();
   void set_field_ptr(Field **field_arg);
   Field **get_next_field_ptr();
   int ping_table_mon_from_table(
@@ -649,7 +717,6 @@ public:
   );
 };
 
-
 struct st_spider_db_request_key
 {
   ulonglong                spider_thread_id;
@@ -783,6 +850,11 @@ public:
     TABLE_LIST *table_list,
     uint table_count
   ) = 0;
+  virtual int reappend_tables(
+    spider_fields *fields,
+    SPIDER_LINK_IDX_CHAIN *link_idx_chain,
+    spider_string *str
+  ) = 0;
   virtual int append_where(
     spider_string *str
   ) = 0;
@@ -1042,6 +1114,10 @@ public:
     Time_zone *time_zone,
     int *need_mon
   ) = 0;
+  virtual bool set_loop_check_in_bulk_sql();
+  virtual int set_loop_check(
+    int *need_mon
+  );
   virtual int fin_loop_check();
   virtual int show_master_status(
     SPIDER_TRX *trx,
@@ -1133,10 +1209,12 @@ public:
   uint dbton_id;
   ha_spider *spider;
   spider_db_share *db_share;
-  /* Index of active server, used in query construction. */
   int first_link_idx;
-  bool strict_group_by= false;
-  bool no_where_cond= false;
+#ifdef SPIDER_HAS_GROUP_BY_HANDLER
+  SPIDER_LINK_IDX_CHAIN *link_idx_chain;
+#endif
+  bool strict_group_by;
+  bool no_where_cond;
   spider_db_handler(ha_spider *spider, spider_db_share *db_share) :
     dbton_id(db_share->dbton_id), spider(spider), db_share(db_share),
     first_link_idx(-1) {}
@@ -1423,6 +1501,9 @@ public:
   virtual int reset_sql(
     ulong sql_type
   ) = 0;
+  virtual bool need_lock_before_set_sql_for_exec(
+    ulong sql_type
+  ) = 0;
 #ifdef SPIDER_HAS_GROUP_BY_HANDLER
   virtual int set_sql_for_exec(
     ulong sql_type,
@@ -1550,6 +1631,10 @@ public:
     spider_fields *fields,
     ulong sql_type
   ) = 0;
+  virtual int reappend_tables_part(
+    spider_fields *fields,
+    ulong sql_type
+  ) = 0;
   virtual int append_where_part(
     ulong sql_type
   ) = 0;
@@ -1693,12 +1778,6 @@ static const LEX_CSTRING maturity_name[] =
   { STRING_WITH_LEN("Stable") }
 };
 
-/*
-  Type of singletons based on the type of the remote database.
-
-  All such singletons are stored in the array `spider_dbton', see
-  `spider_db_init()'.
-*/
 typedef struct st_spider_dbton
 {
   uint dbton_id;
@@ -1785,8 +1864,6 @@ typedef struct st_spider_result_list
     SPIDER_RESULT         *current;
   KEY                     *key_info;
   int                     key_order;
-#ifdef HA_CAN_BULK_ACCESS
-#endif
   spider_string           *sqls;
   int                     ha_read_kind;
   bool                    have_sql_kind_backup;
